Class: Rule
- Inherits:
-
Object
- Object
- Rule
- Defined in:
- lib/Rule.rb
Direct Known Subclasses
Instance Method Summary collapse
-
#initialize ⇒ Rule
constructor
Each rule intitalises itself.
-
#load(value) ⇒ Object
Each rule loads up an array of values.
-
#train ⇒ Object
Each rule trains on values to determine its patterns.
-
#validate ⇒ Object
Each rule compares the data it has with the data it’s given.
Constructor Details
#initialize ⇒ Rule
Each rule intitalises itself.
13 14 15 |
# File 'lib/Rule.rb', line 13 def initialize() @values = nil end |
Instance Method Details
#load(value) ⇒ Object
Each rule loads up an array of values.
18 19 20 |
# File 'lib/Rule.rb', line 18 def load(value) @values << value end |
#train ⇒ Object
Each rule trains on values to determine its patterns.
23 24 25 |
# File 'lib/Rule.rb', line 23 def train() end |
#validate ⇒ Object
Each rule compares the data it has with the data it’s given.
28 29 30 |
# File 'lib/Rule.rb', line 28 def validate() end |